#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#


def evolve(rng, pop, params, recorder=None):
    """
    Evolve a population

    :param rng: random number generator
    :type rng: :class:`fwdpy11.GSLrng`
    :param pop: A population
    :type pop: :class:`fwdpy11.SlocusPop`
    :param params: simulation parameters
    :type params: :class:`fwdpy11.model_params.ModelParams`
    :param recorder: (None) A temporal sampler/data recorder.
    :type recorder: callable

    .. note::
        If recorder is None,
        then :class:`fwdpy11.temporal_samplers.RecordNothing` will be used.

    """
    import fwdpy11.SlocusPop
    import fwdpy11.MlocusPop
    import warnings
    # Test parameters while suppressing warnings
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        # Will throw exception if anything is wrong:
        params.validate()

    from .internal import makeMutationRegions, makeRecombinationRegions
    if pop.__class__ is fwdpy11.SlocusPop:
        from .wright_fisher_slocus import WFSlocusPop
        pneutral = params.mutrate_n/(params.mutrate_n+params.mutrate_s)
        mm = makeMutationRegions(rng, pop, params.nregions,
                                 params.sregions, pneutral)
        rm = makeRecombinationRegions(rng, params.recrate, params.recregions)

        if recorder is None:
            from fwdpy11.temporal_samplers import RecordNothing
            recorder = RecordNothing()

        WFSlocusPop(rng, pop, params.demography,
                    params.mutrate_n, params.mutrate_s,
                    params.recrate, mm, rm, params.gvalue,
                    recorder, params.pself, params.prune_selected)
    else:
        from .wright_fisher_mlocus import WFMlocusPop
        mm = [makeMutationRegions(rng, pop, i, j, n/(n+s)) for
              i, j, n, s in zip(params.nregions,
                                params.sregions,
                                params.mutrates_n, params.mutrates_s)]
        rm = [makeRecombinationRegions(rng, i, j) for i, j in zip(
            params.recrates, params.recregions)]

        if recorder is None:
            from fwdpy11.temporal_samplers import RecordNothing
            recorder = RecordNothing()

        WFMlocusPop(rng, pop, params.demography, params.mutrates_n,
                    params.mutrates_s, mm, rm, params.interlocus_rec,
                    params.gvalue, recorder, params.pself,
                    params.prune_selected)



import unittest
import cppimport
cppimport.force_rebuild()
cppimport.set_quiet(False)
PC = cppimport.imp("pickling_composed_classes")


class testPickleComposedClass(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.D = PC.Derived()
        self.C = PC.Composed(2.0, self.D)

    def testPickle(self):
        import pickle
        p = pickle.dumps(self.C, -1)
        up = pickle.loads(p)
        self.assertEqual(type(up), type(self.C))
        self.assertEqual(type(up.b), type(self.C.b))


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import unittest
import fwdpy11 as fp11


class testMlocusPop(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.pop = fp11.MlocusPop(1000, [(i, i + 1) for i in range(5)])

    def test_N(self):
        self.assertEqual(self.pop.N, 1000)

    def test_nloci(self):
        self.assertEqual(self.pop.nloci, 5)

    def test_fitnesses(self):
        # All fitnesses should be 1
        self.assertEqual(
            sum([i.w for i in self.pop.diploid_metadata]),
            float(self.pop.N))

    def test_labels(self):
        # All diploids should be labeled 0 to pop.N-1
        self.assertEqual(
            [i.label for i in self.pop.diploid_metadata],
            [i for i in range(self.pop.N)])

    def test_genetic_values(self):
        self.assertEqual(sum([i.g for i in self.pop.diploid_metadata]), 0.0)

    def test_e_values(self):
        self.assertEqual(sum([i.e for i in self.pop.diploid_metadata]), 0.0)


class testMlocusPopExceptions(unittest.TestCase):
    def testNzero(self):
        with self.assertRaises(ValueError):
            fp11.MlocusPop(0, [(i, i + 1) for i in range(5)])

    def testNoLoci(self):
        with self.assertRaises(ValueError):
            fp11.MlocusPop(1000, [])


class testSampling(unittest.TestCase):
    @classmethod
    def setUp(self):
        from quick_pops import quick_mlocus_qtrait
        self.pop = quick_mlocus_qtrait()
        self.rng = fp11.GSLrng(42)

    def testRandomSample(self):
        x = self.pop.sample(rng=self.rng, nsam=10)
        # self.assertEqual(len(x), self.pop.nloci)
        # x = self.pop.sample(rng=self.rng, nsam=10, separate=False)
        # self.assertEqual(len(x), self.pop.nloci)
        # x = self.pop.sample(rng=self.rng, nsam=10, remove_fixed=False)
        # self.assertEqual(len(x), self.pop.nloci)
        # x = self.pop.sample(rng=self.rng, nsam=10,
        #                     separate=True, remove_fixed=False)
        # self.assertEqual(len(x), self.pop.nloci)
        # for i in x:
        #     self.assertTrue(isinstance(i, tuple))
        #     self.assertEqual(len(i), 2)
        # x = self.pop.sample(rng=self.rng, nsam=10,
        #                     separate=False, remove_fixed=False)
        # self.assertEqual(len(x), self.pop.nloci)
        # x = self.pop.sample(rng=self.rng, nsam=10,
        #                     separate=True, remove_fixed=True)
        # self.assertEqual(len(x), self.pop.nloci)

    def testDefinedSample(self):
        x = self.pop.sample(individuals=range(10))
        #self.assertEqual(len(x), self.pop.nloci)
        with self.assertRaises(IndexError):
            """
            fwdpp catches case where i >= N
            """
            self.pop.sample(individuals=range(self.pop.N, self.pop.N + 10))

        with self.assertRaises(Exception):
            """
            pybind11 disallows conversion of negative
            numbers to a list of unsigned types.
            """
            self.pop.sample(individuals=range(-10, 10))


class testBadLocusBoundaries(unittest.TestCase):
    def testUnsorted(self):
        with self.assertRaises(ValueError):
            pop = fp11.MlocusPop(100, [(1, 2), (0, 1)])

    def testBadInterval(self):
        with self.assertRaises(ValueError):
            pop = fp11.MlocusPop(100, [(0, 1), (1, 1)])
        with self.assertRaises(ValueError):
            pop = fp11.MlocusPop(100, [(0, 1), (2, 1)])

    def testOverlap(self):
        with self.assertRaises(ValueError):
            pop = fp11.MlocusPop(100, [(0, 1.1), (1, 2)])
        with self.assertRaises(ValueError):
            pop = fp11.MlocusPop(100, [(0, 3), (1, 2)])


class testSettingLocusBoundaries(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.pop = fp11.MlocusPop(100, [(0, 1), (1, 2)])

    def testReassign(self):
        self.pop.locus_boundaries = [(2, 3), (11, 45)]

    def testBadReassign(self):
        with self.assertRaises(ValueError):
            self.pop.locus_boundaries = []
        with self.assertRaises(ValueError):
            self.pop.locus_boundaries = [(1, 2.), (1, 3)]


if __name__ == "__main__":
    unittest.main()



import unittest
import fwdpy11
import fwdpy11.genetic_values
import fwdpy11.ts
import fwdpy11.model_params
import fwdpy11.wright_fisher_ts
import numpy as np


class testTreeSequences(unittest.TestCase):
    @classmethod
    def setUp(self):
        # TODO add neutral variants
        self.N = 1000
        self.demography = np.array([self.N]*self.N, dtype=np.uint32)
        self.rho = 1.
        self.theta = 100.
        self.nreps = 500
        self.mu = self.theta/(4*self.N)
        self.r = self.rho/(4*self.N)

        self.GSS = fwdpy11.genetic_values.GSS(VS=1, opt=0)
        a = fwdpy11.genetic_values.SlocusAdditive(2.0, self.GSS)
        self.p = {'nregions': [],
                  'sregions': [fwdpy11.GaussianS(0, 1, 1, 0.25)],
                  'recregions': [fwdpy11.Region(0, 1, 1)],
                  'rates': (0.0, 0.025, self.r),
                  'gvalue': a,
                  'prune_selected': False,
                  'demography': self.demography
                  }
        self.params = fwdpy11.model_params.ModelParams(**self.p)
        self.rng = fwdpy11.GSLrng(101*45*110*210)
        self.pop = fwdpy11.SlocusPop(self.N, 1.0)
        fwdpy11.wright_fisher_ts.evolve(self.rng, self.pop, self.params, 100)

    def test_dump_to_msprime(self):
        # TODO: test leaf counts of mutations in msprmie
        # vs fwdpy11 and cross-references with self.pop.mcounts
        dumped_ts = self.pop.dump_tables_to_msprime()
        self.assertEqual(len(dumped_ts.tables.nodes),
                         len(self.pop.tables.nodes))
        self.assertEqual(len(dumped_ts.tables.edges),
                         len(self.pop.tables.edges))
        self.assertEqual(len(dumped_ts.tables.mutations),
                         len(self.pop.tables.mutations))
        eview = np.array(self.pop.tables.edges, copy=False)
        self.assertEqual(eview['parent'].sum(),
                         dumped_ts.tables.edges.parent.sum())
        self.assertEqual(eview['child'].sum(),
                         dumped_ts.tables.edges.child.sum())
        self.assertEqual(eview['left'].sum(),
                         dumped_ts.tables.edges.left.sum())
        self.assertEqual(eview['right'].sum(),
                         dumped_ts.tables.edges.right.sum())
        tv = fwdpy11.ts.TreeVisitor(self.pop.tables,
                                    [i for i in range(2*self.pop.N)])
        tt_fwd = 0
        while tv(False) is True:
            m = tv.tree()
            tt_fwd += m.total_time(self.pop.tables.nodes)
        tt_msprime = 0
        for t in dumped_ts.trees():
            tt_msprime += t.get_total_branch_length()
        self.assertEqual(tt_fwd, tt_msprime)

    def test_leaf_counts_vs_mcounts(self):
        tv = fwdpy11.ts.TreeVisitor(self.pop.tables,
                                    [i for i in range(2*self.pop.N)])
        mv = np.array(self.pop.tables.mutations, copy=False)
        muts = np.array(self.pop.mutations.array())
        p = muts['pos']
        while tv(False) is True:
            m = tv.tree()
            l, r = m.left, m.right
            mt = [i for i in mv if p[i[1]] >= l and p[i[1]] < r]
            for i in mt:
                self.assertEqual(m.leaf_counts[i[0]],
                                 self.pop.mcounts[i[1]])

    def test_simplify_to_sample(self):
        """
        Simplify to a sample using fwdpy11.ts and
        msprime, then test that total time on output
        is the same from both sources and that
        the mutation tables contain the same
        positions after simplification.
        """
        dumped_ts = self.pop.dump_tables_to_msprime()
        tt = 0.0
        for i in self.pop.tables.nodes:
            tt += i.time
        samples = np.arange(0, 2*self.pop.N, 50, dtype=np.int32)
        mspts = dumped_ts.simplify(samples=samples.tolist())
        fp11ts, idmap = fwdpy11.ts.simplify(self.pop, samples)
        for i in range(len(fp11ts.edges)):
            self.assertTrue(fp11ts.edges[i].parent < len(fp11ts.nodes))
            self.assertTrue(fp11ts.edges[i].child < len(fp11ts.nodes))
        for s in samples:
            self.assertEqual(
                fp11ts.nodes[idmap[s]].time, self.pop.generation)
        tt_fwd = 0.0
        tv = fwdpy11.ts.TreeVisitor(fp11ts, [i for i in range(len(samples))])
        while tv(False) is True:
            m = tv.tree()
            tt_fwd += m.total_time(fp11ts.nodes)
        tt_msprime = 0.0
        for t in mspts.trees():
            tt_msprime += t.get_total_branch_length()
        self.assertEqual(tt_fwd, tt_msprime)

        self.assertEqual(len(fp11ts.mutations),
                         len(mspts.tables.mutations))
        fp11_pos = np.array([self.pop.mutations[i.key].pos
                             for i in fp11ts.mutations])
        fp11_pos = np.sort(fp11_pos)
        msp_pos = np.sort(mspts.tables.sites.position)
        self.assertTrue(np.array_equal(fp11_pos, msp_pos))

    def test_genotype_matrix(self):
        """
        Make data matrix objects from the tree sequences
        and compare their contents to those of msprime
        as well as to an explicit calculation of mutation counts.
        """
        dm = fwdpy11.ts.make_data_matrix(self.pop,
                                         [i for i in range(2*self.pop.N)],
                                         False, True)
        sa = np.array(dm.selected)
        cs = np.sum(sa, axis=1)
        dumped_ts = self.pop.dump_tables_to_msprime()
        mm = dumped_ts.genotype_matrix()
        mc = np.sum(mm, axis=1)
        ec = np.zeros(len(self.pop.mutations), dtype=np.uint32)
        for d in self.pop.diploids:
            for m in self.pop.gametes[d.first].smutations:
                ec[m] += 1
            for m in self.pop.gametes[d.second].smutations:
                ec[m] += 1
        for i, j, k in zip(self.pop.tables.mutations, cs, mc):
            self.assertEqual(self.pop.mcounts[i.key],
                             ec[i.key])
            self.assertEqual(ec[i.key], j)
            self.assertEqual(ec[i.key], k)
        self.assertTrue(np.array_equal(ec, np.array(self.pop.mcounts)))
        self.assertEqual(ec.sum(), cs.sum())
        self.assertEqual(ec.sum(), mc.sum())
        self.assertTrue(np.array_equal(sa, mm))
        self.assertTrue(np.array_equal(cs, mc))

    def test_VariantIterator(self):
        """
        Test VariantIterator by asserting
        that sum of genotypes equal values in
        the corresponding DataMatrix and
        those in pop.mcounts
        """
        dm = fwdpy11.ts.make_data_matrix(self.pop,
                                         [i for i in range(2*self.pop.N)],
                                         False, True)
        sa = np.array(dm.selected)
        cs = np.sum(sa, axis=1)
        i = 0
        vi = fwdpy11.ts.VariantIterator(self.pop.tables,
                                        self.pop.mutations,
                                        [i for i in range(2*self.pop.N)])
        for v in vi:
            c = self.pop.mcounts[self.pop.tables.mutations[i].key]
            self.assertEqual(c, cs[i])
            self.assertEqual(c, v.genotypes.sum())
            i += 1
        self.assertEqual(i, len(self.pop.tables.mutations))

    def test_VariantIteratorFromPopulation(self):
        dm = fwdpy11.ts.make_data_matrix(self.pop,
                                         [i for i in range(2*self.pop.N)],
                                         False, True)
        sa = np.array(dm.selected)
        cs = np.sum(sa, axis=1)
        i = 0
        vi = fwdpy11.ts.VariantIterator(self.pop)
        for v in vi:
            c = self.pop.mcounts[self.pop.tables.mutations[i].key]
            self.assertEqual(c, cs[i])
            self.assertEqual(c, v.genotypes.sum())
            i += 1
        self.assertEqual(i, len(self.pop.tables.mutations))


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import numpy as np
import msprime


# TODO: mutation origin times are recorded forwards in time,
# which is likely confusing, so we omit them
def _generate_mutation_metadata(pop):
    muts = []
    for mr in pop.tables.mutations:
        m = pop.mutations[mr.key]
        d = {'s': m.s,
             'h': m.h,
             # 'g': m.g,
             'label': m.label,
             'esizes': list(m.esizes),
             'heffects': list(m.heffects),
             'neutral': m.neutral}
        muts.append(str(d).encode('utf-8'))
    return msprime.pack_bytes(muts)


def _initializePopulationTable(node_view, tc):
    population_metadata = []
    for i in sorted(np.unique(node_view['population'])):
        md = "deme"+str(i)
        population_metadata.append(md.encode("utf-8"))

    pmd, pmdo = msprime.pack_bytes(population_metadata)
    tc.populations.set_columns(metadata=pmd, metadata_offset=pmdo)


def _generate_individual_metadata(dmd, tc):
    strings = []
    for i in dmd:
        d = {'g': i.g,
             'e': i.e,
             'w': i.w,
             'geography': i.geography,
             'parents': i.parents,
             'sex': i.sex,
             'deme': i.deme,
             'label': i.label}
        strings.append(str(d).encode("utf-8"))
    return strings


def _initializeIndividualTable(pop, tc):
    """
    Returns node ID -> individual map
    """
    # First, alive individuals:
    individal_nodes = {}
    for i in range(pop.N):
        individal_nodes[2*i] = i
        individal_nodes[2*i+1] = i
    metadata_strings = _generate_individual_metadata(pop.diploid_metadata, tc)

    # Now, preserved nodes
    num_ind_nodes = pop.N
    for i in pop.ancient_sample_records:
        assert i not in individal_nodes, "indivudal record error"
        individal_nodes[i.n1] = num_ind_nodes
        individal_nodes[i.n2] = num_ind_nodes
        num_ind_nodes += 1

    metadata_strings.extend(_generate_individual_metadata(
        pop.ancient_sample_metadata, tc))

    md, mdo = msprime.pack_bytes(metadata_strings)
    flags = [0 for i in range(pop.N+len(pop.ancient_sample_records))]
    tc.individuals.set_columns(flags=flags, metadata=md, metadata_offset=mdo)
    return individal_nodes


def dump_tables_to_msprime(pop):
    """
    Converts fwdpy11.ts.TableCollection to an
    msprime.TreeSequence
    """
    node_view = np.array(pop.tables.nodes, copy=True)
    node_view['time'] -= node_view['time'].max()
    node_view['time'][np.where(node_view['time'] != 0.0)[0]] *= -1.0
    edge_view = np.array(pop.tables.edges, copy=False)
    mut_view = np.array(pop.tables.mutations, copy=False)

    tc = msprime.TableCollection(pop.tables.genome_length())

    # We must initialize population and individual
    # tables before we can do anything else.
    # Attempting to set population to anything
    # other than -1 in an msprime.NodeTable will
    # raise an exception if the PopulationTable
    # isn't set up.
    _initializePopulationTable(node_view, tc)
    node_to_individual = _initializeIndividualTable(pop, tc)

    individual = [-1 for i in range(len(node_view))]
    for k, v in node_to_individual.items():
        individual[k] = v
    flags = [1]*2*pop.N + [0]*(len(node_view) - 2*pop.N)
    tc.nodes.set_columns(flags=flags, time=node_view['time'],
                         population=node_view['population'],
                         individual=individual)
    tc.edges.set_columns(left=edge_view['left'],
                         right=edge_view['right'],
                         parent=edge_view['parent'],
                         child=edge_view['child'])

    mpos = np.array([pop.mutations[i].pos for i in mut_view['key']])
    ancestral_state = np.zeros(len(mut_view), dtype=np.int8)+ord('0')
    ancestral_state_offset = np.arange(len(mut_view)+1, dtype=np.uint32)
    tc.sites.set_columns(position=mpos,
                         ancestral_state=ancestral_state,
                         ancestral_state_offset=ancestral_state_offset)

    derived_state = np.zeros(len(mut_view), dtype=np.int8)+ord('1')
    md, mdo = _generate_mutation_metadata(pop)
    tc.mutations.set_columns(site=np.arange(len(mpos), dtype=np.int32),
                             node=mut_view['node'],
                             derived_state=derived_state,
                             derived_state_offset=ancestral_state_offset,
                             metadata=md,
                             metadata_offset=mdo)
    return tc.tree_sequence()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import unittest

# TODO reimplement to test new API

# import fwdpy11 as fp11
# import fwdpy11.model_params as fp11mp
# import fwdpy11.multilocus as ml
# import fwdpy11.wright_fisher_qtrait as wfq
# from fwdpy11.fitness import SlocusAdditive
# from quick_pops import quick_slocus_qtrait_pop_params
# import numpy as np
# import pickle
# 
# 
# class testSlocusParams(unittest.TestCase):
#     @classmethod
#     def setUpClass(self):
#         # We do not assign certain attributes,
#         # which allows us to test the defaults.
#         self.pdict = {'nregions': [],
#                       'sregions': [],
#                       'recregions': [],
#                       'rates': [0, 0, 0],
#                       'demography': np.array([100] * 100, dtype=np.uint32)
#                       }
# 
#     def test_SlocusParamsDefaults(self):
#         from fwdpy11.fitness import SlocusMult
#         m = fp11mp.SlocusParams(**self.pdict)
#         self.assertEqual(type(m.gvalue), SlocusMult)
# 
#     def test_SlocusParamsQDefaults(self):
#         self.pdict['prune_selected'] = False  # Suppresses a warning
#         from fwdpy11.trait_values import SlocusAdditiveTrait
#         from fwdpy11.wright_fisher_qtrait import GSS
#         m = fp11mp.SlocusParamsQ(**self.pdict)
#         self.assertEqual(type(m.gvalue), SlocusAdditiveTrait)
#         self.assertEqual(type(m.trait2w), GSS)
#         self.assertEqual(m.trait2w.VS, 1.0)
#         self.assertEqual(m.trait2w.O, 0.0)
# 
#     def test_set_nregions(self):
#         self.setUpClass()
#         m = fp11mp.SlocusParams(**self.pdict)
#         try:
#             m.nregions = [fp11.Region(0, 1, 1)]
#         except:
#             self.fail("unexpected exception")
#         # An empty list is ok
#         try:
#             m.nregions = []
#         except:
#             self.fail("unexpected exception")
#         # TypeError b/c not iterable:
#         with self.assertRaises(TypeError):
#             m.nregions = fp11.Region(0, 1, 1)
#         with self.assertRaises(TypeError):
#             m.nregions = [fp11.ExpS(0, 1, 1, 0.25)]
# 
#     def test_set_recregions(self):
#         self.setUpClass()
#         m = fp11mp.SlocusParams(**self.pdict)
#         try:
#             m.recregions = [fp11.Region(0, 1, 1)]
#         except:
#             self.fail("unexpected exception")
#         # An empty list is ok
#         try:
#             m.recregions = []
#         except:
#             self.fail("unexpected exception")
#         # TypeError b/c not iterable:
#         with self.assertRaises(TypeError):
#             m.recregions = fp11.Region(0, 1, 1)
#         with self.assertRaises(TypeError):
#             m.recregions = [fp11.ExpS(0, 1, 1, 0.25)]
# 
#     def test_set_sregions(self):
#         self.setUpClass()
#         m = fp11mp.SlocusParams(**self.pdict)
#         try:
#             m.sregions = [fp11.GaussianS(0, 1, 1, 0.1)]
#         except:
#             self.fail("unexpected exception")
#         # An empty list is ok
#         try:
#             m.sregions = []
#         except:
#             self.fail("unexpected exception")
#         with self.assertRaises(TypeError):
#             m.sregions = fp11.GaussianS(0, 1, 1, 0.2)
#         with self.assertRaises(TypeError):
#             m.sregions = [fp11.Region(0, 1, 1)]
# 
#     def test_init_demog(self):
#         self.setUpClass()
#         try:
#             m = fp11mp.SlocusParams(**self.pdict)
#             m
#         except:
#             self.fail("unexpected exception")
# 
#     def test_init_demog_list(self):
#         self.pdict['demography'] = [100] * 10
#         with self.assertRaises(ValueError):
#             m = fp11mp.SlocusParams(**self.pdict)
#             m
# 
#     def test_init_bad_popsizes(self):
#         self.pdict['demography'] = np.array([100] * 10 + [-1])
#         with self.assertRaises(ValueError):
#             m = fp11mp.SlocusParams(**self.pdict)
#             m
# 
#     def test_bad_rates(self):
#         self.setUpClass()
#         self.pdict['mutrate_n'] = None
#         with self.assertRaises(ValueError):
#             m = fp11mp.SlocusParams(**self.pdict)
#             m
#         self.pdict['mutrate_n'] = -0.01
#         with self.assertRaises(ValueError):
#             m = fp11mp.SlocusParams(**self.pdict)
#             m
# 
#     def test_warning_for_zero_rate(self):
#         # We pass in some sregions but leave mutrate_s == 0.
#         self.setUpClass()
#         self.pdict['sregions'] = [fp11.GammaS(0, 1, 1, -0.1, 0.10, 1.0)]
#         with self.assertWarns(UserWarning):
#             m = fp11mp.SlocusParams(**self.pdict)
#             m.validate()
# 
#     def test_set_rates_list(self):
#         self.setUpClass()
#         self.pdict['rates'] = [0., 1e-3, 0.25]
#         try:
#             m = fp11mp.SlocusParams(**self.pdict)
#             m
#         except:
#             self.fail("unexpected exception")
# 
#     def test_set_rates_tuple(self):
#         m = fp11mp.SlocusParams()
#         m.rates = (0., 1e-3, 0.25)
#         self.assertEqual(m.mutrate_n, 0.0)
#         self.assertEqual(m.mutrate_s, 1e-3)
#         self.assertEqual(m.recrate, 0.25)
#         with self.assertRaises(ValueError):
#             m.rates = (0., -1e-3, 0.25)
# 
#     def test_set_rates_dict(self):
#         m = fp11mp.SlocusParams()
#         with self.assertRaises(ValueError):
#             m.rates = {'mutrate_n': 1e-3}
#         with self.assertRaises(ValueError):
#             m.rates = {'mutrate_n': -1e-3,
#                        'mutrate_s': 1e-3,
#                        'recrate': 1e-3}
#         with self.assertRaises(ValueError):
#             m.rates = {'monkeys_n': 1e-3}
# 
#     def test_set_gvalue(self):
#         m = fp11mp.SlocusParams()
#         from fwdpy11.fitness import SlocusAdditive
#         m.gvalue = SlocusAdditive()
# 
#     def test_set_bad_gvalue(self):
#         m = fp11mp.SlocusParams()
#         with self.assertRaises(ValueError):
#             m.gvalue = []
# 
#     def test_invalid_kwargs(self):
#         from fwdpy11.fitness import SlocusAdditive
#         with self.assertRaises(ValueError):
#             m = fp11mp.SlocusParams(fitness=SlocusAdditive())
#             m
# 
# 
# class testMlocusParams(unittest.TestCase):
#     @classmethod
#     def setUpClass(self):
#         self.rng = fp11.GSLrng(42)
#         nregions = [[fp11.Region(0, 1, 1)], [fp11.Region(1, 2, 1)]]
#         sregions = [
#             [fp11.ExpS(0, 1, 1, -0.1)],
#             [fp11.GammaS(1, 2, 1, -0.1, 0.5)]]
#         recregions = nregions
#         interlocus = ml.binomial_rec([0.5])
#         region_rates = [1e-3, 1e-3]
#         genetic_value = ml.MultiLocusGeneticValue([SlocusAdditive()] * 2)
#         nlist = np.array([100] * 10, dtype=np.uint32)
#         self.param_dict = {'nregions': nregions,
#                            'sregions': sregions,
#                            'recregions': recregions,
#                            'interlocus': interlocus,
#                            'mutrates_n': region_rates,
#                            'mutrates_s': region_rates,
#                            'recrates': region_rates,
#                            'gvalue': genetic_value,
#                            'demography': nlist}
#         self.m = fp11mp.MlocusParams(**self.param_dict)
# 
#     def test_assign_gvalue(self):
#         self.m.gvalue = ml.MultiLocusGeneticValue([SlocusAdditive()] * 2)
# 
#     def test_assign_gvalue_from_list(self):
#         self.m.gvalue = [SlocusAdditive()] * 2
# 
#     def test_assign_gvalue_from_bad_input(self):
#         with self.assertRaises(ValueError):
#             self.m.gvalue = [2.0] * 2
# 
#     def test_assign_aggregator(self):
#         self.m.aggregator = ml.AggAddTrait()
# 
#     def test_assign_aggregator_non_callable(self):
#         with self.assertRaises(ValueError):
#             self.m.aggregator = 2.0
#     # Now, tests that check the validate(), too
# 
#     def test_construction(self):
#         self.setUpClass()
#         try:
#             self.m.validate()
#         except:
#             self.fail("unexpected exception")
# 
#     def test_construction_unequal_region_lens_1(self):
#         self.param_dict['nregions'] = [fp11.Region(0, 1, 1)]
#         with self.assertRaises(TypeError):
#             self.m = fp11mp.MlocusParams(**self.param_dict)
# 
#     def test_construction_unequal_region_lens_2(self):
#         self.param_dict['sregions'] = [fp11.ExpS(0, 1, 1, 0.25)]
#         with self.assertRaises(TypeError):
#             self.m = fp11mp.MlocusParams(**self.param_dict)
# 
#     def test_construction_unequal_region_lens_3(self):
#         self.param_dict['recregions'] = [fp11.Region(0, 1, 1)]
#         with self.assertRaises(TypeError):
#             self.m = fp11mp.MlocusParams(**self.param_dict)
# 
#     def test_empty_sregion_exception(self):
#         self.setUpClass()
#         self.param_dict['sregions'] = [[fp11.ExpS(0, 1, 1, 0.25)], []]
#         self.m = fp11mp.MlocusParams(**self.param_dict)
#         with self.assertRaises(ValueError):
#             self.m.validate()
# 
#     def test_empty_nregion_exception(self):
#         self.setUpClass()
#         self.param_dict['nregions'] = [[], []]
#         self.m = fp11mp.MlocusParams(**self.param_dict)
#         with self.assertRaises(ValueError):
#             self.m.validate()
# 
#     def test_empty_recregion_exception(self):
#         self.setUpClass()
#         self.param_dict['recregions'] = [[fp11.Region(0, 1, 1)], []]
#         self.m = fp11mp.MlocusParams(**self.param_dict)
#         with self.assertRaises(ValueError):
#             self.m.validate()
# 
#     def test_pickle_pdict(self):
#         self.setUpClass()
#         p = pickle.dumps(self.param_dict)
#         up = pickle.loads(p)
#         up
# 
#     def test_pickle_params(self):
#         self.setUpClass()
#         self.m = fp11mp.MlocusParams(**self.param_dict)
#         p = pickle.dumps(self.m)
#         up = pickle.loads(p)
#         up
# 
#     def test_deprecated_keyword(self):
#         self.setUpClass()
#         self.m = fp11mp.MlocusParams(**self.param_dict)
#         with self.assertWarns(DeprecationWarning):
#             self.m.agg = ml.AggAddTrait()
# 
# 
# class testMlocusParamsDefaults(unittest.TestCase):
#     @classmethod
#     def setUpClass(self):
#         nregions = [[fp11.Region(0, 1, 1)], [fp11.Region(1, 2, 1)]]
#         sregions = [
#             [fp11.ExpS(0, 1, 1, -0.1)],
#             [fp11.GammaS(1, 2, 1, -0.1, 0.5)]]
#         recregions = nregions
#         interlocus = ml.binomial_rec([0.5])
#         region_rates = [1e-3, 1e-3]
#         nlist = np.array([100] * 10, dtype=np.uint32)
#         self.pdict = {'nregions': nregions,
#                       'sregions': sregions,
#                       'recregions': recregions,
#                       'interlocus': interlocus,
#                       'mutrates_n': region_rates,
#                       'mutrates_s': region_rates,
#                       'recrates': region_rates,
#                       'demography': nlist
#                       }
# 
#     def test_MlocusParamsDefaults(self):
#         m = fp11mp.MlocusParams(**self.pdict)
#         from fwdpy11.multilocus import AggMultFitness
#         from fwdpy11.multilocus import MultiLocusGeneticValue
#         from fwdpy11.fitness import SlocusMult
#         self.assertEqual(type(m.aggregator), AggMultFitness)
#         self.assertEqual(type(m.gvalue), MultiLocusGeneticValue)
#         for i in m.gvalue.fitness_functions:
#             self.assertEqual(type(i), SlocusMult)
# 
#     def test_MlocusParamsQDefaults(self):
#         self.pdict['prune_selected'] = True
#         m = fp11mp.MlocusParamsQ(**self.pdict)
#         from fwdpy11.multilocus import AggMultTrait
#         from fwdpy11.multilocus import MultiLocusGeneticValue
#         from fwdpy11.trait_values import SlocusAdditiveTrait
#         from fwdpy11.wright_fisher_qtrait import GSS
#         self.assertEqual(type(m.aggregator), AggMultTrait)
#         self.assertEqual(type(m.gvalue), MultiLocusGeneticValue)
#         self.assertEqual(type(m.trait2w), GSS)
#         self.assertEqual(m.trait2w.VS, 1.0)
#         self.assertEqual(m.trait2w.O, 0.0)
#         for i in m.gvalue.fitness_functions:
#             self.assertEqual(type(i), SlocusAdditiveTrait)
# 
#     def test_MlocusParams_cannot_set_gvalue(self):
#         self.pdict.pop('sregions')
#         m = fp11mp.MlocusParams(**self.pdict)
#         self.assertIs(m.gvalue, None)
#         m = fp11mp.MlocusParamsQ(**self.pdict)
#         self.assertIs(m.gvalue, None)
# 
# 
# class test_SlocusParamsQEvolve(unittest.TestCase):
#     @classmethod
#     def setUpClass(self):
#         self.pop, self.pdict = quick_slocus_qtrait_pop_params()
#         self.params = fp11mp.SlocusParamsQ(**self.pdict)
#     def test_evolve(self):
#         rng = fp11.GSLrng(42)
#         wfq.evolve(rng, self.pop, self.params)
# 
# 
# if __name__ == "__main__":
#     pass



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

import unittest
import fwdpy11
import fwdpy11.internal
import numpy as np


class testMutationRegionWeights(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.spop = fwdpy11.SlocusPop(100)
        self.mpop = fwdpy11.MlocusPop(100, [(0, 100), (300, 1000)])
        self.mutrate_n = 1e-5
        self.mutrate_s = 1e-3
        self.pneutral = self.mutrate_n/(self.mutrate_n+self.mutrate_s)
        self.rng = fwdpy11.GSLrng(42)

    def testUniformWeightsSlocus(self):
        """
        Within neutral, selected, all regions are equally-likely to be picked.
        """
        nregions = [fwdpy11.Region(0, 1, 1), fwdpy11.Region(1, 2, 1)]
        sregions = [fwdpy11.ExpS(0, 1, 1, -0.1),
                    fwdpy11.ExpS(1, 2, 1, -0.2),
                    fwdpy11.ConstantS(0, 2, 1, -0.5, 1.0, False)]
        mm = fwdpy11.internal.makeMutationRegions(
            self.rng, self.spop, nregions, sregions, self.pneutral)
        w = np.array(mm.weights)
        pn = w[:2].sum()/w.sum()
        ps = w[2:].sum()/w.sum()
        self.assertEqual(pn, self.pneutral)
        self.assertEqual(ps, 1.0 - self.pneutral)

    def testWeightByRegionSizeSlocus(self):
        """
        Mutation rates are "per base pair", and the total weight that gets assigned 
        to a region is (end-beg)*mutation rate.  Thus Pr(region|mutation) is
        according to a region's length.
        """
        nregion_starts = [0, 100, 5000]
        nregion_stops = [100, 5000, 1e6]  # b/c intervals are [start,stop)
        nregion_lengths = np.array([b-a for a, b in zip(nregion_starts, nregion_stops)],
                                   dtype=np.float64)

        nregions = [fwdpy11.Region(start, stop, self.mutrate_n)
                    for start, stop in zip(nregion_starts, nregion_stops)]

        sregions = [fwdpy11.ExpS(start, stop, self.mutrate_s, -0.1)
                    for start, stop in zip(nregion_starts, nregion_stops)]

        mm = fwdpy11.internal.makeMutationRegions(
            self.rng, self.spop, nregions, sregions, self.pneutral)

        w = np.array(mm.weights)

        pn = w[:len(nregions)].sum()/w.sum()
        ps = w[len(nregions):].sum()/w.sum()

        # We assert almost equal b/c a+b != b+a
        self.assertAlmostEqual(pn, self.pneutral)
        self.assertAlmostEqual(ps, 1.0 - self.pneutral)

        # Check Pr(region x | mutation is neutral)
        nw = w[:len(nregions)]
        for i in range(len(nw)):
            pi = nw[i]/nw.sum()
            ei = nregion_lengths[i]/nregion_lengths.sum()
            self.assertAlmostEqual(pi,ei)

        # Check Pr(region x | mutation is selected)
        sw = w[len(nregions):]
        for i in range(len(nw)):
            pi = sw[i]/sw.sum()
            ei = nregion_lengths[i]/nregion_lengths.sum()
            self.assertAlmostEqual(pi,ei)


if __name__ == "__main__":
    unittest.main()



#See http://cython.readthedocs.io/en/latest/src/tutorial/numpy.html
import numpy as np
cimport numpy as np
DTYPE = np.float
ctypedef np.float_t DTYPE_t
cdef class MeanFitness(object):
    cdef object d
    def __cinit__(self):
        self.d = []
    def __call__(self,pop):
        cdef size_t i = 0
        cdef size_t N = pop.N
        cdef np.ndarray[DTYPE_t,ndim=1] w = np.zeros(N,dtype=DTYPE)
        while i < N:
            w[i]=pop.diploid_metadata[i].w
            i += 1
        self.d.append((pop.generation,w.mean()))
    def get_data(self):
        return self.d



import math


def exponential_size_change(Nstart, Nstop, time):
    """
    Generate a list of population sizes
    according to exponential size_change model

    :param Nstart: population size at onset of size change
    :param Nstop: Population size to reach at end of size change
    :param time: Time (in generations) to get from Nstart to Nstop

    :return: A list of integers representing population size over time.

    .. versionadded:: 0.1.1
    """
    if time < 1:
        raise RuntimeError("time must be >= 1")
    if Nstart < 1 or Nstop < 1:
        raise RuntimeError("Nstart and Nstop must both be >= 1")
    G = math.exp((math.log(Nstop) - math.log(Nstart))/time)
    rv = []
    for i in range(time):
        rv.append(round(Nstart*pow(G, i+1)))
    return rv



import cppimport
import fwdpy11
import fwdpy11.ezparams
import fwdpy11.model_params
import fwdpy11.wright_fisher
import fwdpy11.fwdpy11_types
import pickle
import unittest
cppimport.force_rebuild()
ca = cppimport.imp("custom_additive")
general = cppimport.imp("custom_stateless_genotype")


class testCustomAdditive(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.pop = fwdpy11.SlocusPop(1000)
        self.pdict = fwdpy11.ezparams.mslike(self.pop,
                                             dfe=fwdpy11.ExpS(0, 1, 1, -0.05),
                                             pneutral=0.95, simlen=10)
        self.pdict['gvalue'] = ca.additive()
        self.rng = fwdpy11.GSLrng(42)
        self.params = fwdpy11.model_params.ModelParams(**self.pdict)

    def testEvolve(self):
        fwdpy11.wright_fisher.evolve(self.rng, self.pop, self.params)

    def testPickle(self):
        a = self.params.gvalue
        p = pickle.dumps(a, -1)
        up = pickle.loads(p)
        self.assertEqual(type(a), type(up))

    # TODO: test this once built-in SlocusAdditive is callable
    # def testCorrectNess(self):
    #     fwdpy11.wright_fisher.evolve(self.rng, self.pop, self.params)
    #     a = fwdpy11.fitness.SlocusAdditive(2.0)
    #     for i in self.pop.diploids:
    #         self.assertEqual(i.w, a(i, self.pop))


class testGeneralModule(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.pop = fwdpy11.SlocusPop(1000)
        self.pdict = fwdpy11.ezparams.mslike(self.pop,
                                             dfe=fwdpy11.ConstantS(
                                                 0, 1, 1, -0.05, 0.05),
                                             pneutral=0.95, simlen=10)
        self.pdict['gvalue'] = general.GeneralW()
        self.rng = fwdpy11.GSLrng(42)
        self.params = fwdpy11.model_params.ModelParams(**self.pdict)

    def testPickle(self):
        a = self.params.gvalue
        p = pickle.dumps(a, -1)
        up = pickle.loads(p)
        self.assertEqual(type(a), type(up))

    def testEvolve(self):
        fwdpy11.wright_fisher.evolve(self.rng, self.pop, self.params)


if __name__ == "__main__":
    unittest.main()



# Unit tests for fwdpy11.regions

import fwdpy11
import unittest


class test_Region_repr(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.nregion = fwdpy11.Region(0, 1, 1)
        self.constants = fwdpy11.ConstantS(0, 1, 1, -0.1, 2.0)
        self.uniforms = fwdpy11.UniformS(0, 1, 1, 1, 2, 0)
        self.exps = fwdpy11.ExpS(0, 1, 1, 0.25)
        self.gaussians = fwdpy11.GaussianS(0, 1, 1, 0.25)
        self.gammas = fwdpy11.GammaS(0, 1, 1, 0.124, 2.0)

    def test_nregion(self):
        x = eval('fwdpy11.' + repr(self.nregion))

    def test_constants(self):
        x = eval('fwdpy11.' + repr(self.constants))

    def test_uniforms(self):
        x = eval('fwdpy11.' + repr(self.uniforms))

    def test_exps(self):
        x = eval('fwdpy11.' + repr(self.exps))

    def test_gaussians(self):
        x = eval('fwdpy11.' + repr(self.gaussians))

    def test_gammas(self):
        x = eval('fwdpy11.' + repr(self.gammas))


class test_Scaling(unittest.TestCase):
    def test_ConstantSwithoutScaling(self):
        """
        Here, the scaling is 1, and the DFE is constant.
        Thus, all "s" values must be equal to 10.0
        """
        x = fwdpy11.ConstantS(0, 1, 1, s=10, scaling=1)
        c = x.callback()
        rng = fwdpy11.GSLrng(42)
        sh = c(rng)
        self.assertEqual(sh[0], 10.0)

    def test_ConstantSwithScaling(self):
        """
        Here, the scaling is 100, and the DFE is constant.
        Thus, all "s" values must be equal to 10.0/100.0,
        or 0.1.
        """
        x = fwdpy11.ConstantS(0, 1, 1, s=10, scaling=100)
        c = x.callback()
        rng = fwdpy11.GSLrng(42)
        sh = c(rng)
        self.assertEqual(sh[0], 0.1)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpye1.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

# This is part of fwdpy11's test suite.
# This file contains functions for quickly
# obtaining simulated populations


def quick_neutral_slocus(N=1000, simlen=100):
    from fwdpy11.ezparams import mslike
    from fwdpy11.model_params import ModelParams
    from fwdpy11 import SlocusPop, GSLrng
    from fwdpy11.genetic_values import SlocusMult
    from fwdpy11.wright_fisher import evolve
    pop = SlocusPop(N)
    params_dict = mslike(pop, simlen=simlen)
    params_dict['gvalue'] = SlocusMult(2.)
    params = ModelParams(**params_dict)
    rng = GSLrng(42)
    evolve(rng, pop, params)
    return pop


def quick_nonneutral_slocus(N=1000, simlen=100, dfe=None):
    from fwdpy11.ezparams import mslike
    from fwdpy11.model_params import ModelParams
    from fwdpy11 import SlocusPop, GSLrng
    from fwdpy11.wright_fisher import evolve
    from fwdpy11 import ExpS
    from fwdpy11.genetic_values import SlocusMult
    pop = SlocusPop(N)
    if dfe is None:
        dfe = ExpS(0, 1, 1, -0.1)
    params_dict = mslike(
        pop, simlen=simlen, dfe=dfe,
        pneutral=0.95)
    params_dict['gvalue'] = SlocusMult(2.0)
    params = ModelParams(**params_dict)
    rng = GSLrng(42)
    evolve(rng, pop, params)
    return pop


def quick_slocus_qtrait_pop_params(N=1000, simlen=100):
    from fwdpy11 import SlocusPop
    from fwdpy11.genetic_values import GSS
    from fwdpy11.genetic_values import SlocusAdditive
    from fwdpy11 import GaussianS, Region
    import numpy as np
    p = {'nregions': [],
         'sregions': [GaussianS(0, 1, 1, 0.25)],
         'recregions': [Region(0, 1, 1)],
         'rates': (0.0, 2e-3, 1e-3),
         'demography': np.array([N] * simlen, dtype=np.uint32),
         'gvalue': SlocusAdditive(2.0, GSS(VS=1.0, opt=0.0)),
         'prune_selected': False
         }
    pop = SlocusPop(N)
    return (pop, p)


def quick_mlocus_qtrait_pop_params(N=1000, simlen=100):
    from fwdpy11 import MlocusPop
    from fwdpy11.genetic_values import GSS
    from fwdpy11.genetic_values import MlocusAdditive
    from fwdpy11 import GaussianS, Region
    from fwdpy11.multilocus import binomial_rec
    import numpy as np

    theta, rho = 100., 100.
    mu = 1e-3
    sigmu = 0.25
    nloci = 5
    # Simulate a nloci locus system where each locus is 10 units long
    locus_boundaries = [(float(i + i * 10), float(i + i * 10 + 10))
                        for i in range(nloci)]
    nregions = [[Region(j[0], j[1], theta / (4. * float(N)), coupled=True)]
                for i, j in zip(range(nloci), locus_boundaries)]
    recregions = [[Region(j[0], j[1], rho / (4. * float(N)), coupled=True)]
                  for i, j in zip(range(nloci), locus_boundaries)]
    sregions = [[GaussianS(j[0] + 5., j[0] + 6., mu, sigmu, coupled=False)]
                for i, j in zip(range(nloci), locus_boundaries)]
    interlocus_rec = binomial_rec([0.5] * (nloci - 1))
    nlist = np.array([N] * simlen, dtype=np.uint32)
    mutrates_n = [theta / (4. * float(N))] * nloci
    mutrates_s = [mu] * nloci
    recrates = [rho / (4. * float(N))] * nloci
    param_dict = {'nregions': nregions,
                  'sregions': sregions,
                  'recregions': recregions,
                  'rates': (mutrates_n, mutrates_s, recrates),
                  'interlocus_rec': interlocus_rec,
                  'gvalue': MlocusAdditive(2.0, GSS(VS=1.0, opt=0.0)),
                  'demography': nlist,
                  'prune_selected': False
                  }
    pop = MlocusPop(N, locus_boundaries)
    return (pop, param_dict)


def quick_mlocus_qtrait(N=1000, simlen=100):
    from fwdpy11 import GSLrng
    from fwdpy11.wright_fisher import evolve
    from fwdpy11.model_params import ModelParams
    rng = GSLrng(42)
    pop, param_dict = quick_mlocus_qtrait_pop_params(N, simlen)
    params = ModelParams(**param_dict)
    evolve(rng, pop, params)
    return pop


# TODO: fix or remove
def quick_mlocus_qtrait_change_optimum(N=1000,
                                       simlen=100, prune_selected=False):
    """
    .. warning:: May result in long-running tests
    """
    from fwdpy11.model_params import ModelParams
    from fwdpy11.genetic_values import MlocusAdditive
    from fwdpy11.genetic_values import GSSmo
    from fwdpy11 import MlocusPop, GSLrng
    from fwdpy11.wright_fisher import evolve
    from fwdpy11 import GaussianS, Region
    from fwdpy11.multilocus import binomial_rec
    import numpy as np

    rng = GSLrng(42)
    theta, rho = 100., 100.
    mu = 1e-3
    sigmu = 0.25
    nloci = 5
    # Simulate a nloci locus system where each locus is 10 units long
    locus_boundaries = [(float(i + i * 10), float(i + i * 10 + 10))
                        for i in range(nloci)]
    nregions = [[Region(j[0], j[1], theta / (4. * float(N)), coupled=True)]
                for i, j in zip(range(nloci), locus_boundaries)]
    recregions = [[Region(j[0], j[1], rho / (4. * float(N)), coupled=True)]
                  for i, j in zip(range(nloci), locus_boundaries)]
    sregions = [[GaussianS(j[0] + 5., j[0] + 6., mu, sigmu, coupled=False)]
                for i, j in zip(range(nloci), locus_boundaries)]
    interlocus_rec = binomial_rec([0.5] * (nloci - 1))
    nlist = np.array([N] * simlen, dtype=np.uint32)
    param_dict = {'nregions': nregions,
                  'sregions': sregions,
                  'recregions': recregions,
                  'interlocus': interlocus_rec,
                  'mutrates_n': [theta / (4. * float(N))] * nloci,
                  'mutrates_s': [mu] * nloci,
                  'recrates': [rho / (4. * float(N))] * nloci,
                  'gvalue': MlocusAdditive(2.0, GSSmo([(0, 0, 1),
                                                       (simlen / 2, 1, 1)])),
                  'demography': nlist,
                  'prune_selected': prune_selected}
    params = ModelParams(**param_dict)
    pop = MlocusPop(N, locus_boundaries)
    evolve(rng, pop, params)
    return pop



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import math


class Region(object):
    """
    Representation of a "region" in a simulation.

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * l: A label assigned to the region.  Labels must be integers,
            and can be used to 'tag' mutations arising in different regions.

    See :func:`fwdpy11.wright_fisher.evolve` for how this class may be used to
        parameterize a simulation.

    This class is extended by:
        * :class:`fwdpy11.Sregion`
    """

    def __init__(self, beg, end, weight, coupled=True, label=0):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used to take mutations
            from this region.

        When coupled is True, the "weight" may be interpreted
        as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            r = fwdpy11.Region(0,1,1)
            #A more "biological" case:
            #  The region covers positions 1 through 1,000,
            #  and the per-base pair "weight" is 1e-5:
            r = fwdpy11.Region(1,1000,1e-5,True)
        """
        self.b = float(beg)
        self.e = float(end)
        self.w = float(weight)
        self.c = coupled
        self.l = label
        if self.c is True:
            self.w = (self.e - self.b) * self.w

    @property
    def b(self):
        """
        Beginning of a region
        """
        return self.__b

    @b.setter
    def b(self, beg):
        if math.isinf(beg):
            raise ValueError("fwdpy11.Region: beg not finite")
        if math.isnan(beg):
            raise ValueError("fwdpy11.Region: beg not a number")
        self.__b = beg

    @property
    def e(self):
        """
        End of a region
        """
        return self.__e

    @e.setter
    def e(self, end):
        if math.isnan(end):
            raise ValueError("fwdpy11.Region: end not a number")
        if math.isinf(end):
            raise ValueError("fwdpy11.Region: end not finite")
        self.__e = end

    @property
    def w(self):
        """
        Weight on a region.
        """
        return self.__w

    @w.setter
    def w(self, weight):
        if math.isinf(weight):
            raise ValueError("fwdpy11.Region: weight not finite")
        if math.isnan(weight):
            raise ValueError("fwdpy11.Region: weight not a number")
        if weight < 0.0:
            raise ValueError("fwdpy11.Region: weight < 0.0")
        self.__w = weight

    @property
    def c(self):
        """
        If weight is coupled to end-beg. (boolean)
        """
        return self.__c

    @c.setter
    def c(self, coupled):
        self.__c = coupled

    @property
    def l(self):
        """
        Region label.
        """
        return self.__l

    @l.setter
    def l(self, label):
        import numpy as np
        self.__l = np.uint16(label)

    def __repr__(self):
        x = 'Region(beg=%s,end=%s,'
        x += 'weight=%s,coupled=%s,label=%s)'
        return x % (self.b, self.e, self.w, self.c, self.l)


class Sregion(Region):
    """
    Representation of a "region" in a simulation with a dominance term.

    This class is the base class for a general set
    of objects representing distributions of fitness effects.

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * h: the dominance term
        * l: A label assigned to the region.
            Labels must be integers, and can be
            used to 'tag' mutations arising in different regions.
        * scaling: The scaling of the distrubution.  See note below.

    See :func:`fwdpy11.wright_fisher.evolve` for how this class
    may be used to parameterize a simulation.

    .. note:: This class cannot be used directly to parameterize a simulation.
        Rather, you must use a derived type that specifies a
        distribution of fitness effects.  These types include:
       :class:`fwdpy11.ConstantS`,
       :class:`fwdpy11.UniformS`,
       :class:`fwdpy11.ExpS`,
       :class:`fwdpy11.GammaS`, and
       :class:`fwdpy11.GaussianS`

    .. note:: The scaling of a distribution refers to the distribution of effect sizes.
        For example, if scaling = 1.0, then the DFE is on the effect size itself.  If
        scaling = 2N (where N is the population size), then the DFE is on 2Ns.  If N
        is not constant during a simulation, then the scaling is with respect to some
        "reference" population size.

    .. versionchanged:: 0.13.a2
        Added "scaling" attribute.

    """

    def __init__(self, beg, end, weight, h=1.0,
                 coupled=True, label=0, scaling=1):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param h: the dominance
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used to take mutations
            from this region.
        :param scaling: The scaling of the DFE

        When coupled is True, the "weight" may be
        interpreted as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            #Examples for models where the 3 genotype fitnesses are
            #1, 1+sh, and 1+2s, respectively
            recessive = fwdpy11.Sregion(0,1,1,0)
            additive = fwdpy11.Sregion(0,1,1,1.0)
            dominant = fwdpy11.Sregion(0,1,1,2.0)
        """
        super(Sregion, self).__init__(beg, end, weight, coupled, label)
        self.h = h
        self.scaling = scaling
        self.__des = None

    def callback(self):
        """
        Return the C++ callback associated with this Sregion.
        """
        return self.des

    @property
    def h(self):
        """
        Dominance (h)
        """
        return self.__h

    @h.setter
    def h(self, h):
        if math.isinf(h):
            raise ValueError("fwdpy11.Sregion: h not finite")
        if math.isnan(h):
            raise ValueError("fwdpy11.Segion: h not a number")
        self.__h = float(h)

    @property
    def scaling(self):
        """
        Scaling term for the DFE
        """
        return self.__scaling

    @scaling.setter
    def scaling(self, scaling):
        import numpy as np
        self.__scaling = np.uint32(scaling)

    @property
    def des(self):
        """
        Distribution of effect sizes.
        """
        import fwdpy11.fwdpp_extensions
        return getattr(fwdpy11.fwdpp_extensions, self.__des[0])(*self.__des[1])

    @des.setter
    def des(self, des):
        try:
            import fwdpy11.fwdpp_extensions
            if callable(getattr(fwdpy11.fwdpp_extensions, des[0])) is False:
                raise ValueError(
                    "callable required for distribution of effect sizes")
            getattr(fwdpy11.fwdpp_extensions, des[0])(*des[1])
        except:
            raise
        self.__des = des

    # def __str__(self):
    #    retval = "h = " + "{:.9f}".format(self.h)
    #    retval += ", " + super(Sregion, self).__str__()


class GammaS(Sregion):
    """
    Gamma distribution of fitness effects

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * mean: mean of the Gamma
        * shape: shape of the Gamma
        * h: the dominance term
        * l: A label assigned to the region.
            Labels must be integers, and can be used to
            'tag' mutations arising in different regions.
        * scaling: The scaling of the distrubution.

    See :func:`fwdpy11.wright_fisher.evolve` for how this
    class may be used to parameterize a simulation
    """

    def __init__(self, beg, end, weight, mean, shape, h=1.0,
                 coupled=True, label=0, scaling=1):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param mean: the mean selection coefficient
        :param shape: the shape parameter of the distribution
        :param h: the dominance
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used
            to take mutations from this region.
        :param scaling: The scaling of the DFE

        When coupled is True, the "weight" may be
        interpreted as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            gdist = fwdpy11.GammaS(0,1,1,-0.1,0.35)
        """
        super(GammaS, self).__init__(
            beg, end, weight, h, coupled, label, scaling)
        self.mean = float(mean)
        self.shape = float(shape)
        self.des = ("makeGammaSH", (self.mean,
                                    self.shape, self.h, self.scaling))

    @property
    def mean(self):
        """
        Mean of the distribution.
        """
        return self.__mean

    @mean.setter
    def mean(self, mean):
        if math.isinf(mean):
            raise ValueError("fwdpy11.GammaS: mean not finite")
        if math.isnan(mean):
            raise ValueError("fwdpy11.GammaS: mean not a number")
        self.__mean = float(mean)

    @property
    def shape(self):
        """
        Shape parameter.
        """
        return self.__shape

    @shape.setter
    def shape(self, shape):
        if math.isinf(shape):
            raise ValueError("fwdpy11.GammaS: shape not finite")
        if math.isnan(shape):
            raise ValueError("fwdpy11.GammaS: shape not a number")
        self.__shape = float(shape)

    def __repr__(self):
        x = 'GammaS(beg=%s,end=%s,weight=%s,'
        x += 'mean=%s,shape=%s,coupled=%s,label=%s,scaling=%s)'
        return x % (self.b, self.e, self.w, self.mean,
                    self.shape, self.c, self.l, self.scaling)


class ConstantS(Sregion):
    """
    Constant/fixed selection coefficient

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * s: the selection coefficient
        * h: the dominance term
        * l: A label assigned to the region.
            Labels must be integers, and can be used to
            'tag' mutations arising in different regions.
        * scaling: The scaling of the distrubution.

    See :func:`fwdpy11.wright_fisher.evolve` for how this class may be used to
    parameterize a simulation
    """

    def __init__(self, beg, end, weight, s, h=1.0, coupled=True, label=0, scaling=1):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param s: the selection coefficient
        :param h: the dominance
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used to take mutations
            from this region.
        :param scaling: The scaling of the DFE

        When coupled is True, the "weight" may be interpreted
        as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            #s = -0.1 and h = 0
            constantS = fwdpy11.ConstantS(0,1,1,-0.1,0)
        """
        super(ConstantS, self).__init__(
            beg, end, weight, h, coupled, label, scaling)
        self.s = s
        self.des = ("makeConstantSH", (self.s, self.h, self.scaling))

    @property
    def s(self):
        """
        Effect size.
        """
        return self.__s

    @s.setter
    def s(self, s):
        if math.isinf(s):
            raise ValueError("fwdpy11.ConstantS: s not finite")
        if math.isnan(s):
            raise ValueError("fwdpy11.ConstantS: s not a number")
        self.__s = float(s)

    def __repr__(self):
        x = 'ConstantS(beg=%s,end=%s,weight=%s,'
        x += 's=%s,h=%s,coupled=%s,label=%s,scaling=%s)'
        return x % (self.b, self.e, self.w, self.s, self.h, self.c, self.l, self.scaling)


class UniformS(Sregion):
    """
    Uniform distribution on selection coefficients

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * lo: the lower bound on s
        * hi: the upper bound on s
        * h: the dominance term
        * l: A label assigned to the region.
            Labels must be integers, and can be used to
            'tag' mutations arising in different regions.
        * scaling: The scaling of the distrubution.

    See :func:`fwdpy11.wright_fisher.evolve` for how this
    class may be used to parameterize a simulation
    """

    def __init__(self, beg, end, weight, lo, hi, h=1.0, coupled=True, label=0, scaling=1):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param lo: lower bound on s
        :param hi: upper bound on s
        :param h: the dominance
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used to take
            mutations from this region.
        :param scaling: The scaling of the DFE

        When coupled is True, the "weight" may be
        interpreted as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            #s is uniform on [-1, 0]
            uniformS = fwdpy11.UniformS(0,1,1,-1,0,0)
        """
        super(UniformS, self).__init__(beg, end, weight, h, coupled, scaling)
        self.lo = lo
        self.hi = hi
        self.des = ("makeUniformSH", (self.lo, self.hi, self.h, self.scaling))

    @property
    def lo(self):
        """
        Lo value for effect size.
        """
        return self.__lo

    @lo.setter
    def lo(self, lo):
        if math.isinf(lo):
            raise ValueError("fwdpy11.UniformS: lo not finite")
        if math.isnan(lo):
            raise ValueError("fwdpy11.UniformS: lo not a number")
        self.__lo = float(lo)

    @property
    def hi(self):
        """
        Hi value for effect size.
        """
        return self.__hi

    @hi.setter
    def hi(self, hi):
        if math.isinf(hi):
            raise ValueError("fwdpy11.UniformS: hi not finite")
        if math.isnan(hi):
            raise ValueError("fwdpy11.UniformS: hi not a number")
        self.__hi = float(hi)

    def __repr__(self):
        x = 'UniformS(beg=%s,end=%s,weight=%s,'
        x += 'lo=%s,hi=%s,h=%s,coupled=%s,label=%s,scaling=%s)'
        return x % (self.b, self.e, self.w, self.lo,
                    self.hi, self.h, self.c, self.l, self.scaling)


class ExpS(Sregion):
    """
    Exponential distribution on selection coefficients

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * mean: the mean selection coefficient
        * h: the dominance term
        * l: A label assigned to the region.
            Labels must be integers, and can be used to
            'tag' mutations arising in different regions.
        * scaling: The scaling of the distrubution.

    See :func:`fwdpy11.wright_fisher.evolve` for how this class may be used to
    parameterize a simulation
    """

    def __init__(self, beg, end, weight, mean, h=1.0, coupled=True, label=0, scaling=1):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param mean: the mean selection coefficient
        :param h: the dominance
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used to take
            mutations from this region.
        :param scaling: The scaling of the DFE

        When coupled is True, the "weight" may be
        interpreted as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            #s is exp(-0.1) and recessive
            expS = fwdpy11.ExpS(0,1,1,-0.1,0)
        """
        super(ExpS, self).__init__(
            beg, end, weight, h, coupled, label, scaling)
        self.mean = mean
        self.des = ("makeExpSH", (self.mean, self.h, self.scaling))

    @property
    def mean(self):
        """
        Mean of the distribution
        """
        return self.__mean

    @mean.setter
    def mean(self, mean):
        if math.isinf(mean):
            raise ValueError("fwdpy11.ExpS: mean not finite")
        if math.isnan(mean):
            raise ValueError("fwdpy11.ExpS: mean not a number")
        self.__mean = float(mean)

    def __repr__(self):
        x = 'ExpS(beg=%s,end=%s,weight=%s,'
        x += 'mean=%s,h=%s,coupled=%s,label=%s,scaling=%s)'
        return x % (self.b, self.e, self.w, self.mean, self.h, self.c, self.l, self.scaling)


class GaussianS(Sregion):
    """
    Gaussian distribution on selection coefficients
    (effect sizes for sims of quantitative traits)

    Attributes:
        * b: the beginning of the region
        * e: the end of the region
        * w: the "weight" assigned to the region
        * sd: the standard deviation
        * h: the dominance term
        * l: A label assigned to the region.
            Labels must be integers, and can be used to
            'tag' mutations arising in different regions.
        * scaling: The scaling of the distrubution.

    The mean is zero.

    See :func:`fwdpy11.wright_fisher.evolve` for how this class may be used to
    parameterize a simulation
    """

    def __init__(self, beg, end, weight, sd, h=1.0, coupled=True, label=0, scaling=1):
        """
        Constructor

        :param beg: the beginning of the region
        :param end: the end of the region
        :param weight: the weight to assign
        :param sd: standard deviation of effect sizes 
        :param h: the dominance
        :param coupled: if True, the weight is converted to (end-beg)*weight
        :param label: Not relevant to recombining regions.
            Otherwise, this value will be used to take mutations
            from this region.
        :param scaling: The scaling of the DFE

        When coupled is True, the "weight" may be
        interpreted as a "per base pair"
        (or per unit, generally speaking) term.

        Example:

        .. testcode::

            #A simple case
            import fwdpy11
            #s N(0,0.1) and co-dominant
            gaussianS = fwdpy11.GaussianS(0,1,1,0.1,1)
        """
        super(GaussianS, self).__init__(
            beg, end, weight, h, coupled, label, scaling)
        self.sd = sd
        self.des = ("makeGaussianSH", (self.sd, self.h, self.scaling))

    @property
    def sd(self):
        """
        Standard deviation of the distribution
        """
        return self.__sd

    @sd.setter
    def sd(self, sd):
        if math.isinf(sd):
            raise ValueError("fwdpy11.GaussianS: sd not finite")
        if math.isnan(sd):
            raise ValueError("fwdpy11.GaussianS: sd not a number")
        self.__sd = float(sd)

    def __repr__(self):
        x = 'GaussianS(beg=%s,end=%s,weight=%s,'
        x += 'sd=%s,h=%s,coupled=%s,label=%s,scaling=%s)'
        return x % (self.b, self.e, self.w, self.sd, self.h, self.c, self.l, self.scaling)



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#


def makeMutationRegions(rng, pop, neutral, selected, pneutral):
    """
    Convert user input into :class:`~fwdpy11.fwdpp_extensions.MutationRegions`

    :param rng: A :class:`fwdpy11.GSLrng`
    :param pop: A :class:`fwdp11.Population`
    :param neutral: A list of :class:`fwdpy11.regions.Region` objects.
    :param selected: A list of :class:`fwdpy11.regions.Sregion` objects.
    :param pneutral: The probability that a new mutation is neutral
    :ptype neutral: float

    :rtype: :class:`fwdpy11.fwdpp_extensions.MutationRegions`

    .. note:: Used by various "evolve" functions.
        Users probably won't need to call this.

        >>> import fwdpy11 as fp11
        >>> nregions = [fp11.Region(0,0.5,1),fp11.Region(1,1.5,1)]
        >>> sregions = [fp11.ExpS(0,0.5,1,1),fp11.GaussianS(1,1.5,1,0.25)]
        >>> mr = fp11.makeMutationRegions(nregions,sregions,0.1)
        >>> type(mr)
        <class 'fwdpy11.fwdpp_extensions.MutationRegions'>

    One or both lists may be empty:

        >>> mr = fp11.makeMutationRegions([],[],0.0)

    Neither list may be None:

        >>> mr = fp11.makeMutationRegions([],None,0.0)
        Traceback (most recent call last):
            ...
        TypeError: 'NoneType' object is not iterable

    .. versionchanged:: 0.2.0
        Added pneutral to handle changes in fwdpp 0.6.0
    """
    import math
    if math.isfinite(pneutral) is False:
        raise ValueError("pneutral not finite")
    elif pneutral < 0.0 or pneutral > 1.0:
        raise ValueError("pneutral must be in the range [0.0, 1.0]")
    import numpy as np

    # We need to reweight the user input so that new
    # mutations come out at the expected rates.
    # This is necessary b/c the user inputs weights
    # *separately* for neutral and selected variants.
    # These weights have to get combined into a single
    # weight vector on the C++ side, meaning that we
    # have some normalization to do:
    neutral_region_weights = np.array([i.w for i in neutral], dtype=np.float64)
    neutral_region_weights /= neutral_region_weights.sum()
    neutral_region_weights *= pneutral
    selected_region_weights = np.array([i.w for i in selected], dtype=np.float64)
    selected_region_weights /= selected_region_weights.sum()
    selected_region_weights *= (1.0-pneutral)
    summed_weights = neutral_region_weights.sum() + selected_region_weights.sum()
    neutral_region_weights /= summed_weights
    selected_region_weights /= summed_weights
    ntuples = [(i.b, i.e, j, i.l)
               for i, j in zip(neutral, neutral_region_weights)]
    stuples = [(i.b, i.e, j, i.l)
               for i, j in zip(selected, selected_region_weights)]
    callbacks = [i.callback() for i in selected]
    from .fwdpp_extensions import MutationRegions
    return MutationRegions(rng, pop, ntuples, stuples, callbacks)


def makeRecombinationRegions(rng, recrate, regions):
    """
    Convert user input into
    :class:`~fwdpy11.fwdpp_extensions.RecombinationRegions`

    :param rng: :class:`fwdpy11.GSLrng`
    :param recrate: (float) Recombination reate.
    :param regions: A list of :class:`fwdpy11.Region`

    :rtype: :class:`fwdpy11.fwdpp_extensions.RecombinationRegions`


    .. note::
        Used by various "evolve" functions.
        Users probably won't need to call this.

        >>> import fwdpy11 as fp11
        >>> rng = fwdpy11.GSLrng(42)
        >>> rregions = [fp11.Region(0,0.5,1),fp11.Region(1,1.5,1)]
        >>> rr = fp11.makeRecombinationRegions(rng, 1e-3, rregions)
        >>> type(rr)
        <class 'fwdpy11.fwdpp_extensions.RecombinationRegions'>

    """
    beg = [i.b for i in regions]
    end = [i.e for i in regions]
    weights = [i.w for i in regions]
    from .fwdpp_extensions import RecombinationRegions
    return RecombinationRegions(rng, recrate, beg, end, weights)



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

# Unit tests of fwdpy11.util

import unittest
from quick_pops import quick_neutral_slocus, quick_mlocus_qtrait
import fwdpy11
import fwdpy11.util


class testAddMutations(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.pop = quick_neutral_slocus()
        self.rng = fwdpy11.GSLrng(42)


class test_ChangeEsizeSlocus(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.pop = quick_neutral_slocus()
        self.rng = fwdpy11.GSLrng(42)

    def test_change_esize(self):
        # Get extant mutations
        extant = [i for i in enumerate(self.pop.mcounts) if i[1] > 0]
        # Get gametes with this mutation
        g = [i[0] for i in enumerate(
            self.pop.gametes) if extant[0][0] in i[1].mutations and
            self.pop.gametes[i[0]].n > 0]
        self.assertEqual(sum([self.pop.gametes[i].n for i in g]), extant[0][1])
        fwdpy11.util.change_effect_size(self.pop, extant[0][0], -0.1)
        self.assertEqual(self.pop.mutations[extant[0][0]].s, -0.1)
        self.assertEqual(self.pop.mutations[extant[0][0]].neutral, False)
        g2 = [i[0] for i in enumerate(self.pop.gametes) if
              extant[0][0] in i[1].smutations and
              self.pop.gametes[i[0]].n > 0]
        self.assertEqual(g, g2)

    def test_change_esize_nothing_happens(self):
        """
        Same test as above, but we change s from 0 to 0
        """
        # Get extant mutations
        extant = [i for i in enumerate(self.pop.mcounts) if i[1] > 0]
        # Get gametes with this mutation
        g = [i[0] for i in enumerate(
            self.pop.gametes) if extant[0][0] in i[1].mutations and
            self.pop.gametes[i[0]].n > 0]
        self.assertEqual(sum([self.pop.gametes[i].n for i in g]), extant[0][1])
        fwdpy11.util.change_effect_size(self.pop, extant[0][0], 0.0)
        self.assertEqual(self.pop.mutations[extant[0][0]].s, 0.0)
        self.assertEqual(self.pop.mutations[extant[0][0]].neutral, True)
        g2 = [i[0] for i in enumerate(self.pop.gametes) if
              extant[0][0] in i[1].mutations and
              self.pop.gametes[i[0]].n > 0]
        self.assertEqual(g, g2)

    def testVectorEffects(self):
        extant = [i for i in enumerate(self.pop.mcounts) if i[1] > 0]
        g = [i[0] for i in enumerate(
            self.pop.gametes) if extant[0][0] in i[1].mutations and
            self.pop.gametes[i[0]].n > 0]
        fwdpy11.util.change_effect_size(self.pop, extant[0][0],
                                        0.0, 0.0, [1.], [-1.])
        self.assertEqual(self.pop.mutations[extant[0][0]].neutral, False)
        self.assertEqual(self.pop.mutations[extant[0][0]].esizes[0], 1.0)
        self.assertEqual(self.pop.mutations[extant[0][0]].heffects[0], -1.0)

        # Now, replace it w/a neutral mutation
        fwdpy11.util.change_effect_size(self.pop, extant[0][0],
                                        0.0, 0.0, [0.], [-1.])
        self.assertEqual(self.pop.mutations[extant[0][0]].neutral, True)
        self.assertEqual(self.pop.mutations[extant[0][0]].esizes[0], 0.0)
        self.assertEqual(self.pop.mutations[extant[0][0]].heffects[0], -1.0)


class test_ChangeEsizeMlocus(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.pop = quick_mlocus_qtrait()

    def test_change_esize(self):
        extant = [i for i in enumerate(self.pop.mcounts) if i[1] > 0]
        # Get gametes with this mutation
        g = [i[0] for i in enumerate(
            self.pop.gametes) if extant[0][0] in i[1].mutations and
            self.pop.gametes[i[0]].n > 0]
        self.assertEqual(sum([self.pop.gametes[i].n for i in g]), extant[0][1])
        fwdpy11.util.change_effect_size(self.pop, extant[0][0], -0.1)
        self.assertEqual(self.pop.mutations[extant[0][0]].s, -0.1)
        self.assertEqual(self.pop.mutations[extant[0][0]].neutral, False)
        g2 = [i[0] for i in enumerate(self.pop.gametes) if
              extant[0][0] in i[1].smutations and
              self.pop.gametes[i[0]].n > 0]
        self.assertEqual(g, g2)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
# Test behavior of fixations in simulations.
# As of 0.1.3a1, fixations are to be sorted by position
# for all types of simulations, allowing for easier lookup.
#
# In 0.2.0, this unit test was refactored to skip simulation.
# Instead, we use pybind11 + cppimport to expose the underlying
# C++ API to Python, create pops manually, and test the API.

import unittest
import os
import fwdpy11
import cppimport
cppimport.force_rebuild()
fp = cppimport.imp("fixation_properties")


class testFixationsAreSortedSlocusPop(unittest.TestCase):
    @classmethod
    def setUp(self):
        mutations = fwdpy11.VecMutation()
        fixations = fwdpy11.VecMutation()
        gametes = fwdpy11.VecGamete()
        diploids = fwdpy11.VecDiploid()
        mutations.append(fwdpy11.Mutation(0.1, -0.01, 1.0, 0, 0))
        mutations.append(fwdpy11.Mutation(0.2, 0.0, 1.0, 0, 0))
        gametes.append(fwdpy11.Gamete(
            (4, fwdpy11.VecUint32([1]), fwdpy11.VecUint32([0]))))
        diploids.append(fwdpy11.DiploidGenotype(0, 0))
        diploids.append(fwdpy11.DiploidGenotype(0, 0))
        self.pop = fwdpy11.SlocusPop(diploids, gametes, mutations)

    def testSetup(self):
        self.assertEqual(len(self.pop.mutations), 2)
        self.assertEqual(len(self.pop.gametes), 1)
        self.assertEqual(len(self.pop.diploids), 2)
        for g in self.pop.gametes:
            self.assertEqual(len(g.mutations), 1)
            self.assertEqual(len(g.smutations), 1)
        for mc in self.pop.mcounts:
            self.assertEqual(mc, 4)

    def testWithPruneSelected(self):
        fp.update_mutations(self.pop, 1, 2 * len(self.pop.diploids), True)
        self.assertEqual(len(self.pop.fixations), 2)
        self.assertEqual(len(self.pop.fixation_times), 2)
        self.assertEqual(len([i for i in self.pop.mcounts if i == 0]), 2)
        self.assertTrue(sorted(self.pop.fixations, key=lambda x: x.pos))
        for mc in self.pop.mcounts:
            self.assertEqual(mc, 0)

    def testWithoutPruneSelected(self):
        fp.update_mutations(self.pop, 1, 2 * len(self.pop.diploids), False)
        self.assertEqual(len(self.pop.fixations), 2)
        self.assertEqual(len(self.pop.fixation_times), 2)
        self.assertEqual(len([i for i in self.pop.mcounts if i == 0]), 1)
        self.assertTrue(sorted(self.pop.fixations, key=lambda x: x.pos))
        for m, mc in zip(self.pop.mutations, self.pop.mcounts):
            if m.neutral is True:
                self.assertEqual(mc, 0)
            else:
                self.assertEqual(mc, 4)


class testFixationsAreSortedMlocusPop(unittest.TestCase):
    @classmethod
    def setUp(self):
        mutations = fwdpy11.VecMutation()
        fixations = fwdpy11.VecMutation()
        gametes = fwdpy11.VecGamete()
        diploids = fwdpy11.VecVecDiploid()
        mutations.append(fwdpy11.Mutation(0.25, -0.01, 1.0, 0, 0))
        mutations.append(fwdpy11.Mutation(0.75, 0.0, 1.0, 0, 0))
        mutations.append(fwdpy11.Mutation(1.25, 0.0, 1.0, 0, 0))
        mutations.append(fwdpy11.Mutation(1.75, 0.25, 1.0, 0, 0))
        gametes.append(fwdpy11.Gamete(
            (4, fwdpy11.VecUint32([1]), fwdpy11.VecUint32([0]))))
        gametes.append(fwdpy11.Gamete(
            (4, fwdpy11.VecUint32([2]), fwdpy11.VecUint32([3]))))
        dip = fwdpy11.VecDiploid(
            [fwdpy11.DiploidGenotype(0, 0), fwdpy11.DiploidGenotype(1, 1)])
        diploids.append(dip)
        diploids.append(dip)
        self.pop = fwdpy11.MlocusPop(
            diploids, gametes, mutations, [(0, 1), (1, 2)])

    def testSetup(self):
        self.assertEqual(len(self.pop.mutations), 4)
        self.assertEqual(len(self.pop.gametes), 2)
        self.assertEqual(len(self.pop.diploids), 2)
        for d in self.pop.diploids:
            self.assertEqual(len(d), 2)
            for l in d:
                self.assertEqual(len(self.pop.gametes[l.first].mutations), 1)
                self.assertEqual(len(self.pop.gametes[l.first].smutations), 1)
                self.assertEqual(len(self.pop.gametes[l.second].mutations), 1)
                self.assertEqual(len(self.pop.gametes[l.second].smutations), 1)

        self.assertEqual(len(self.pop.locus_boundaries), 2)
        self.assertEqual(self.pop.nloci, 2)
        for g in self.pop.gametes:
            self.assertEqual(len(g.mutations), 1)
            self.assertEqual(len(g.smutations), 1)
        for mc in self.pop.mcounts:
            self.assertEqual(mc, 4)

    def testWithPruneSelected(self):
        fp.update_mutations(self.pop, 1, 2 * len(self.pop.diploids), True)
        self.assertEqual(len(self.pop.fixations), 4)
        self.assertEqual(len(self.pop.fixation_times), 4)
        self.assertEqual(len([i for i in self.pop.mcounts if i == 0]), 4)
        self.assertTrue(sorted(self.pop.fixations, key=lambda x: x.pos))
        for mc in self.pop.mcounts:
            self.assertEqual(mc, 0)

    def testWithoutPruneSelected(self):
        fp.update_mutations(self.pop, 1, 2 * len(self.pop.diploids), False)
        self.assertEqual(len(self.pop.fixations), 4)
        self.assertEqual(len(self.pop.fixation_times), 4)
        self.assertEqual(len([i for i in self.pop.mcounts if i == 0]), 2)
        self.assertTrue(sorted(self.pop.fixations, key=lambda x: x.pos))
        for m, mc in zip(self.pop.mutations, self.pop.mcounts):
            if m.neutral is True:
                self.assertEqual(mc, 0)
            else:
                self.assertEqual(mc, 4)


if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#

import sys

if sys.version_info[0] < 3:
    raise ValueError("Python3 required!")

from fwdpy11._init import * # NOQA
from fwdpy11._version import __version__ # NOQA

from .fwdpp_types import *
from ._opaque_gametes import *
from ._opaque_mutations import *
from ._opaque_diploids import *
from ._Population import VecUint32
from .fwdpy11_types import DiploidGenotype
from .fwdpy11_types import DiploidMetadata
from .fwdpy11_types import Mutation
from ._regions import *
from ._dev import *
from ._gslrng import GSLrng
from ._Population import Population
from .SlocusPop import SlocusPop
from .MlocusPop import MlocusPop




#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#
import cppimport
cppimport.force_rebuild()
cppimport.set_quiet(False)
pickling_cpp = cppimport.imp("pickling_cpp")
import unittest


class testPickleMutation(unittest.TestCase):
    @classmethod
    def setUp(self):
        import fwdpy11
        self.m = fwdpy11.Mutation(0.1, -0.2, 1.0, 10, 3)

    def testUnpickle(self):
        import pickle
        o = pickling_cpp.pickle_mutation(self.m)
        m = pickle.loads(o)
        self.assertEqual(m, self.m)

    def testUnpickleFromGeneralPickler(self):
        import pickle
        o = pickling_cpp.general_pickler(self.m)
        m = pickle.loads(o)
        self.assertEqual(m, self.m)


class testPickleSlocusPop(unittest.TestCase):
    @classmethod
    def setUp(self):
        from quick_pops import quick_neutral_slocus
        self.pop = quick_neutral_slocus()

    def testPickleMutations(self):
        import pickle
        p = [pickling_cpp.general_pickler(i) for i in self.pop.mutations]
        for i, j in zip(p, self.pop.mutations):
            m = pickle.loads(i)
            self.assertEqual(m, j)
        x = pickle.dumps(self.pop.mutations)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.mutations):
            self.assertEqual(i, j)

    def testPickleGametes(self):
        import pickle
        p = [pickling_cpp.general_pickler(i) for i in self.pop.gametes]
        for i, j in zip(p, self.pop.gametes):
            m = pickle.loads(i)
            self.assertEqual(m, j)
        x = pickling_cpp.general_pickler(self.pop.gametes)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.gametes):
            self.assertEqual(i, j)

    def testPickleDiploidsPy(self):
        import pickle
        pd = pickle.dumps(self.pop.diploids)
        p = pickle.loads(pd)
        for i, j in zip(p, self.pop.diploids):
            self.assertEqual(i, j)

    def testPickleDiploidsCpp(self):
        import pickle
        p = [pickling_cpp.general_pickler(i) for i in self.pop.diploids]
        for i, j in zip(p, self.pop.diploids):
            m = pickle.loads(i)
            self.assertEqual(m, j)
        x = pickling_cpp.general_pickler(self.pop.diploids)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.diploids):
            self.assertEqual(i, j)

    def testPicklePopPy(self):
        import pickle
        p = pickle.dumps(self.pop, -1);
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)

    def testPicklePopCpp(self):
        import pickle
        p = pickling_cpp.general_pickler(self.pop)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)


class testPickleMlocusPop(unittest.TestCase):
    @classmethod
    def setUp(self):
        from quick_pops import quick_mlocus_qtrait
        self.pop = quick_mlocus_qtrait(N=500, simlen=10)

    def testPickleDiploids(self):
        import pickle
        x = pickle.dumps(self.pop.diploids)
        p = pickle.loads(x)
        for i, j in zip(p, self.pop.diploids):
            self.assertEqual(i, j)

    def testPicklePopPy(self):
        import pickle
        p = pickle.dumps(self.pop, -1)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)

    def testPicklePopCpp(self):
        import pickle
        p = pickling_cpp.general_pickler(self.pop)
        pp = pickle.loads(p)
        self.assertEqual(pp, self.pop)



if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017 Kevin Thornton <krthornt@uci.edu>
#
# This file is part of fwdpy11.
#
# fwdpy11 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# fwdpy11 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with fwdpy11.  If not, see <http://www.gnu.org/licenses/>.
#


def evolve(rng, pop, params, simplification_interval, recorder=None,
           suppress_table_indexing=False):
    """
    Evolve a population

    :param rng: random number generator
    :type rng: :class:`fwdpy11.GSLrng`
    :param pop: A population
    :type pop: :class:`fwdpy11.SlocusPop`
    :param params: simulation parameters
    :type params: :class:`fwdpy11.model_params.ModelParams`
    :param simplification_interval: Number of generations between simplifications.
    :type simplification_interval: int
    :param recorder: (None) A temporal sampler/data recorder.
    :type recorder: callable
    :param suppress_table_indexing: (False) Prevents edge table indexing until end of simulation
    :type suppress_table_indexing: boolean

    .. note::
        If recorder is None,
        then :class:`fwdpy11.tsrecorders.NoAncientSamples` will be used.

    """
    import warnings

    # Currently, we do not support simulating neutral mutations
    # during tree sequence simulations, so we make sure that there
    # are no neutral regions/rates:
    if len(params.nregions) != 0:
        raise ValueError(
            "Simulation of neutral mutations on tree sequences not supported (yet).")

    # Test parameters while suppressing warnings
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        # Will throw exception if anything is wrong:
        params.validate()

    from .internal import makeMutationRegions, makeRecombinationRegions
    from .wright_fisher_slocus_ts import WFSlocusPop_ts
    # TODO: update to allow neutral mutations
    pneutral = 0
    mm = makeMutationRegions(rng, pop, params.nregions,
                             params.sregions, pneutral)
    rm = makeRecombinationRegions(rng, params.recrate, params.recregions)

    if recorder is None:
        from fwdpy11.tsrecorders import NoAncientSamples
        recorder = NoAncientSamples()

    from fwdpy11.tsrecorders import SampleRecorder
    sr = SampleRecorder()
    WFSlocusPop_ts(rng, pop, sr, simplification_interval,
                   params.demography, params.mutrate_s,
                   params.recrate, mm, rm, params.gvalue,
                   recorder, params.pself, params.prune_selected is True,
                   suppress_table_indexing)
